package cmd

import (
	"testing"
	"time"

	"github.com/stretchr/testify/assert"
)

func TestBuildInfo_ParseBuildTime_RFC3339(t *testing.T) {
	// GIVEN
	formatted := "2024-05-17T04:45:42+03:00"

	// WHEN
	buildTime := parseBuildTime(formatted)

	// THEN
	assert.Equal(t, 2024, buildTime.Year())
	assert.Equal(t, time.May, buildTime.Month())
	assert.Equal(t, 17, buildTime.Day())
	assert.Equal(t, 4, buildTime.Hour())
	assert.Equal(t, 45, buildTime.Minute())
	assert.Equal(t, 42, buildTime.Second())
	_, offset := buildTime.Zone()
	assert.Equal(t, 3*60*60, offset)
}

func TestBuildInfo_ParseBuildTime_RFC3339ish(t *testing.T) {
	// GIVEN
	formatted := "2024-05-17T04:45:42+0300"

	// WHEN
	buildTime := parseBuildTime(formatted)

	// THEN
	assert.Equal(t, 2024, buildTime.Year())
	assert.Equal(t, time.May, buildTime.Month())
	assert.Equal(t, 17, buildTime.Day())
	assert.Equal(t, 4, buildTime.Hour())
	assert.Equal(t, 45, buildTime.Minute())
	assert.Equal(t, 42, buildTime.Second())
	_, offset := buildTime.Zone()
	assert.Equal(t, 3*60*60, offset)
}

func TestBuildInfo_ParseBuildTime_InvalidFormat(t *testing.T) {
	// GIVEN
	formatted := "2024-05-17T04"

	// WHEN / THEN
	assert.Panics(t, func() { parseBuildTime(formatted) })
}
